package org.bitbucket.bradleysmithllc.webserviceshubclient.parameter;

import org.apache.commons.io.FileUtils;
import org.bitbucket.bradleysmithllc.webserviceshubclient.regexp.ParameterExpression;
import org.bitbucket.bradleysmithllc.webserviceshubclient.regexp.ScopeExpression;

import java.io.*;
import java.util.*;

public class ParameterFile
{
	private final List<ScopeSection> sections = new ArrayList<ScopeSection>();
	private final Map<String, ScopeSection> sectionMap = new HashMap<String, ScopeSection>();

	public ParameterFile(File prmFile) throws IOException
	{
		initialize(FileUtils.readFileToString(prmFile));
	}

	public ParameterFile(String contents) throws IOException
	{
		initialize(contents);
	}

	public ParameterFile()
	{
	}

	private void initialize(String prmFile) throws IOException
	{
		BufferedReader bread = new BufferedReader(new StringReader(prmFile));

		try
		{
			String line;

			while ((line = bread.readLine()) != null)
			{
				// check for a scope change
				ScopeExpression se = new ScopeExpression(line);

				if (se.matches())
				{
					appendSection(se);
				}
				// ignore everything until a section is defined
				else if (sections.size() > 0)
				{
					// look for a parameter
					ParameterExpression pe = new ParameterExpression(line);

					if (pe.matches())
					{
						appendParameter(pe);
					}
				}
			}
		}
		finally
		{
			bread.close();
		}
	}

	public void appendParameter(ParameterExpression pe)
	{
		if (sections.size() == 0)
		{
			throw new IllegalStateException("No sections defined");
		}

		sections.get(sections.size() - 1).appendParameter(pe);
	}

	public void appendParameter(Parameter pe)
	{
		if (sections.size() == 0)
		{
			throw new IllegalStateException("No sections defined");
		}

		sections.get(sections.size() - 1).appendParameter(pe);
	}

	public ScopeSection appendSection(ScopeExpression se)
	{
		return appendSection(new ScopeSectionImpl(se));
	}

	public ScopeSection appendSection(String str)
	{
		ScopeExpression se = new ScopeExpression(str);

		if (se.matches())
		{
			return appendSection(se);
		}
		else
		{
			throw new IllegalArgumentException("Bad scope '" + str + "'");
		}
	}

	public ScopeSection appendSection(ScopeSection ss)
	{
		if (sectionMap.containsKey(ss.getHeader()))
		{
			throw new IllegalArgumentException("Section: " + ss.getHeader() + " already added");
		}

		sections.add(ss);
		sectionMap.put(ss.getHeader(), ss);

		return ss;
	}

	public ScopeSection getSection(String scope)
	{
		return sectionMap.get(scope);
	}

	public Map<String, ScopeSection> getSectionMap()
	{
		return sectionMap;
	}

	public List<ScopeSection> getSections()
	{
		return sections;
	}

	public void write(File output) throws IOException
	{
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(output)));

		try
		{
			write(pw);
		}
		finally
		{
			pw.close();
		}
	}

	public void write(PrintWriter output) throws IOException
	{
		output.println("# Parameter file generated by webserviceshubclient 1.6.0 on " + new Date());

		for (ScopeSection section : sections)
		{
			output.println();

			section.write(output);
		}
	}
}